@@ -40,6 +40,7 @@ use ws::{Message, Sender, WebSocket};
 
 use errors::{Error as ZolaError, Result};
 use globset::GlobSet;
+use pathdiff::diff_paths;
 use relative_path::{RelativePath, RelativePathBuf};
 use site::sass::compile_sass;
 use site::{Site, SITE_CONTENT};
@@ -72,6 +73,7 @@ static NOT_FOUND_TEXT: &[u8] = b"Not Found";
 const LIVE_RELOAD: &str = include_str!("livereload.js");
 
 async fn handle_request(req: Request<Body>, mut root: PathBuf) -> Result<Response<Body>> {
+    let original_root = root.clone();
     let mut path = RelativePathBuf::new();
     // https://zola.discourse.group/t/percent-encoding-for-slugs/736
     let decoded = match percent_encoding::percent_decode_str(req.uri().path()).decode_utf8() {
@@ -111,6 +113,11 @@ async fn handle_request(req: Request<Body>, mut root: PathBuf) -> Result<Respons
     // otherwise `PathBuf` will interpret it as an absolute path
     root.push(&decoded[1..]);
 
+    // Ensure we are only looking for things in our public folder
+    if !root.starts_with(original_root) {
+        return Ok(not_found());
+    }
+
     let metadata = match tokio::fs::metadata(root.as_path()).await {
         Err(err) => return Ok(io_error(err)),
         Ok(metadata) => metadata,
@@ -300,12 +307,14 @@ pub fn serve(
         return Err(format!("Cannot start server on address {}.", address).into());
     }
 
-    let config_path = config_file.to_str().unwrap_or("config.toml");
+    let config_path = PathBuf::from(config_file);
+    let config_path_rel = diff_paths(&config_path, &root_dir).unwrap_or(config_path.clone());
 
-    // An array of (path, bool, bool) where the path should be watched for changes, and the boolean value
-    // indicates whether this file/folder must exist for zola serve to operate
+    // An array of (path, WatchMode) where the path should be watched for changes,
+    // and the WatchMode value indicates whether this file/folder must exist for
+    // zola serve to operate
     let watch_this = vec![
-        (config_path, WatchMode::Required),
+        (config_path_rel.to_str().unwrap_or("config.toml"), WatchMode::Required),
         ("content", WatchMode::Required),
         ("sass", WatchMode::Condition(site.config.compile_sass)),
         ("static", WatchMode::Optional),
@@ -518,7 +527,7 @@ pub fn serve(
                         );
 
                         let start = Instant::now();
-                        match detect_change_kind(root_dir, &path, config_path) {
+                        match detect_change_kind(root_dir, &path, &config_path) {
                             (ChangeKind::Content, _) => {
                                 console::info(&format!("-> Content changed {}", path.display()));
 
@@ -644,13 +653,10 @@ fn is_temp_file(path: &Path) -> bool {
 
 /// Detect what changed from the given path so we have an idea what needs
 /// to be reloaded
-fn detect_change_kind(pwd: &Path, path: &Path, config_filename: &str) -> (ChangeKind, PathBuf) {
+fn detect_change_kind(pwd: &Path, path: &Path, config_path: &Path) -> (ChangeKind, PathBuf) {
     let mut partial_path = PathBuf::from("/");
     partial_path.push(path.strip_prefix(pwd).unwrap_or(path));
 
-    let mut partial_config_path = PathBuf::from("/");
-    partial_config_path.push(config_filename);
-
     let change_kind = if partial_path.starts_with("/templates") {
         ChangeKind::Templates
     } else if partial_path.starts_with("/themes") {
@@ -661,7 +667,7 @@ fn detect_change_kind(pwd: &Path, path: &Path, config_filename: &str) -> (Change
         ChangeKind::StaticFiles
     } else if partial_path.starts_with("/sass") {
         ChangeKind::Sass
-    } else if partial_path == partial_config_path {
+    } else if path == config_path {
         ChangeKind::Config
     } else {
         unreachable!("Got a change in an unexpected path: {}", partial_path.display());
@@ -709,43 +715,43 @@ mod tests {
                 (ChangeKind::Templates, PathBuf::from("/templates/hello.html")),
                 Path::new("/home/vincent/site"),
                 Path::new("/home/vincent/site/templates/hello.html"),
-                "config.toml",
+                Path::new("/home/vincent/site/config.toml"),
             ),
             (
                 (ChangeKind::Themes, PathBuf::from("/themes/hello.html")),
                 Path::new("/home/vincent/site"),
                 Path::new("/home/vincent/site/themes/hello.html"),
-                "config.toml",
+                Path::new("/home/vincent/site/config.toml"),
             ),
             (
                 (ChangeKind::StaticFiles, PathBuf::from("/static/site.css")),
                 Path::new("/home/vincent/site"),
                 Path::new("/home/vincent/site/static/site.css"),
-                "config.toml",
+                Path::new("/home/vincent/site/config.toml"),
             ),
             (
                 (ChangeKind::Content, PathBuf::from("/content/posts/hello.md")),
                 Path::new("/home/vincent/site"),
                 Path::new("/home/vincent/site/content/posts/hello.md"),
-                "config.toml",
+                Path::new("/home/vincent/site/config.toml"),
             ),
             (
                 (ChangeKind::Sass, PathBuf::from("/sass/print.scss")),
                 Path::new("/home/vincent/site"),
                 Path::new("/home/vincent/site/sass/print.scss"),
-                "config.toml",
+                Path::new("/home/vincent/site/config.toml"),
             ),
             (
                 (ChangeKind::Config, PathBuf::from("/config.toml")),
                 Path::new("/home/vincent/site"),
                 Path::new("/home/vincent/site/config.toml"),
-                "config.toml",
+                Path::new("/home/vincent/site/config.toml"),
             ),
             (
                 (ChangeKind::Config, PathBuf::from("/config.staging.toml")),
                 Path::new("/home/vincent/site"),
                 Path::new("/home/vincent/site/config.staging.toml"),
-                "config.staging.toml",
+                Path::new("/home/vincent/site/config.staging.toml"),
             ),
         ];
 
@@ -760,7 +766,7 @@ mod tests {
         let expected = (ChangeKind::Templates, PathBuf::from("/templates/hello.html"));
         let pwd = Path::new(r#"C:\\Users\johan\site"#);
         let path = Path::new(r#"C:\\Users\johan\site\templates\hello.html"#);
-        let config_filename = "config.toml";
+        let config_filename = Path::new(r#"C:\\Users\johan\site\config.toml"#);
         assert_eq!(expected, detect_change_kind(pwd, path, config_filename));
     }
 
@@ -769,7 +775,7 @@ mod tests {
         let expected = (ChangeKind::Templates, PathBuf::from("/templates/hello.html"));
         let pwd = Path::new("/home/johan/site");
         let path = Path::new("templates/hello.html");
-        let config_filename = "config.toml";
+        let config_filename = Path::new("config.toml");
         assert_eq!(expected, detect_change_kind(pwd, path, config_filename));
     }
 }
